package cn.org.bjca.trust.hm.demo.slice;

import cn.org.bjca.trust.hm.demo.ResourceTable;
import cn.org.bjca.trust.hm.demo.common.GsonImplHelp;
import com.huawei.hms.location.harmony.*;
import ohos.aafwk.content.Intent;
import ohos.agp.components.Switch;
import ohos.agp.components.Text;
import ohos.hiviewdfx.HiLog;

public class LocationAbilitySlice extends BaseAbilitySlice {
    private static final String TAG = "LocationAbilitySlice";

    private Text showLocation;
    private SettingsProviderClient settingsProviderClient;
    private LocationRequest locationRequest;
    private int number = 0;

    @Override
    public void onStart(Intent intent) {
        super.onStart(intent);
        super.setUIContent(ResourceTable.Layout_ability_main);
        checkSelfPermission();
        showLocation = (Text) findComponentById(ResourceTable.Id_text_helloworld);
        showLocation.setClickedListener(component -> {
        });
        settingsProviderClient = new SettingsProviderClient(this);

        findComponentById(ResourceTable.Id_get_settings).setClickedListener(v -> {
            checkLocationSettings();
        });
        initRequest();
        ((Switch) findComponentById(ResourceTable.Id_request)).setCheckedStateChangedListener((absButton, b) -> {
            if (b) {
                number = 1;
                fusedLocationClient.requestLocationUpdates(locationRequest, locationCallback)
                        .addOnSuccessListener(v -> {
                            showLocation.setText("持续获取位置信息，开启完成");
                        })
                        .addOnFailureListener(e -> {
                            showLocation.setText("持续获取位置信息，开启失败：" + GsonImplHelp.get().toJson(e));
                        });
            } else {
                // 注意：停止位置更新时，mLocationCallback与requestLocationUpdates()中的LocationCallback参数为同一对象。
                fusedLocationClient.removeLocationUpdates(locationCallback)
                        .addOnSuccessListener(v -> {
                            showLocation.setText("持续获取位置信息，关闭完成");
                        })
                        .addOnFailureListener(e -> {
                            showLocation.setText("持续获取位置信息，关闭失败：" + GsonImplHelp.get().toJson(e));
                        });
            }
        });
        findComponentById(ResourceTable.Id_get_last).setClickedListener(v -> {

            getLastLocation();
        });
        findComponentById(ResourceTable.Id_get_last_address).setClickedListener(v -> {
            fusedLocationClient.getLastLocationWithAddress(buildLocationRequest())
                    .addOnSuccessListener(location -> {
                        if (null == location) {
                            return;
                        }
                        showLocation.setText("获取最后一次位置信息完成：" + GsonImplHelp.get().toJson(location));
                    })
                    .addOnFailureListener(e -> {
                        showLocation.setText("获取最后一次位置信息失败：" + GsonImplHelp.get().toJson(e));
                    });

        });
    }

    private void initRequest() {
        locationRequest = new LocationRequest();
// 设置位置更新的间隔（毫秒：单位）
        locationRequest.setInterval(2000);
// 设置权重
        locationRequest.setPriority(LocationRequest.PRIORITY_BALANCED_POWER_ACCURACY);
// （可选）设置是否需要返回地址信息
        locationRequest.setNeedAddress(true);
// （可选）设置返回地址信息的语言
        locationRequest.setLanguage("zh");
    }

    private LocationCallback locationCallback = new LocationCallback() {
        @Override
        public void onLocationResult(LocationResult locationResult) {
            if (locationResult != null) {
                showLocation.setText("持续获取位置信息，第" + number + "次：" + GsonImplHelp.get().toJson(locationResult));
                number += 1;
            }
        }

        @Override
        public void onLocationAvailability(LocationAvailability locationAvailability) {
            super.onLocationAvailability(locationAvailability);
            if (locationAvailability != null) {
                // 处理位置状态
            }
        }
    };

    private void checkLocationSettings() {
        LocationRequest locationRequest = new LocationRequest();
        locationRequest.setPriority(100);
        LocationSettingsRequest.Builder builder = new LocationSettingsRequest.Builder();
        LocationSettingsRequest request =
                builder.addLocationRequest(locationRequest).setAlwaysShow(false).setNeedBle(false).build();
        settingsProviderClient.checkLocationSettings(request)
                .addOnSuccessListener(response -> {
                    // 设置成功定位条件
                    showLocation.setText("成功：" + GsonImplHelp.get().toJson(response));
                })
                .addOnFailureListener(exp -> {
                    // 设置不满足定位条件
                    showLocation.setText("失败：" + GsonImplHelp.get().toJson(exp));
                });
    }

    private void getLastLocation() {
        showLocation.setText("logs：");
        fusedLocationClient.getLastLocation().addOnSuccessListener(location -> {
            if (null == location) {
                printLog(HiLog.INFO, TAG, "[old]getLastLocation onSuccess: location is null");
                return;
            }
            String result = "获取最后有效位置成功 onSuccess location[Longitude,Latitude]:" + location.getLongitude() + "," + location.getLatitude();
            printLog(HiLog.INFO, TAG, result);
            printScreenLog(showLocation, result);
        }).addOnFailureListener(e -> {
            printLog(HiLog.INFO, TAG, "获取最后有效位置失败:" + e.getMessage());
            printScreenLog(showLocation, "获取最后有效位置失败:" + e.getMessage());
        });
    }

    private LocationRequest buildLocationRequest() {
        LocationRequest locationRequest = new LocationRequest();
        locationRequest.setPriority(LocationRequest.PRIORITY_BALANCED_POWER_ACCURACY);
        locationRequest.setInterval(5000);
        locationRequest.setNeedAddress(true);
        locationRequest.setLanguage("zh");
        return locationRequest;
    }

    @Override
    public void onActive() {
        super.onActive();
    }

    @Override
    public void onForeground(Intent intent) {
        super.onForeground(intent);
    }
}
