package cn.org.bjca.trust.hm.demo.slice;

import cn.org.bjca.trust.hm.demo.ResourceTable;
import ohos.aafwk.ability.AbilitySlice;
import ohos.aafwk.ability.DataAbilityHelper;
import ohos.aafwk.content.Intent;
import ohos.aafwk.content.Operation;
import ohos.agp.components.Image;
import ohos.hiviewdfx.HiLog;
import ohos.hiviewdfx.HiLogLabel;
import ohos.media.image.ImageSource;
import ohos.media.image.PixelMap;
import ohos.media.photokit.metadata.AVStorage;
import ohos.utils.net.Uri;

import java.io.File;
import java.io.FileDescriptor;

public class MediumAbilitySlice extends AbilitySlice {
    private static final HiLogLabel LABEL_LOG = new HiLogLabel(HiLog.LOG_APP, 0xD001234, "MediumAbilitySlice");

    private Image img;
    private String photoCameraPath;

    @Override
    public void onStart(Intent intent) {
        super.onStart(intent);
        super.setUIContent(ResourceTable.Layout_ability_medium);
        findComponentById(ResourceTable.Id_get_images).setClickedListener(component -> {
            Intent intent1 = new Intent();
            intent1.setType("image/*");
            Operation opt = new Intent.OperationBuilder().withAction("android.intent.action.GET_CONTENT").build();
            intent1.setOperation(opt);
            intent1.addFlags(Intent.FLAG_NOT_OHOS_COMPONENT);

            startAbilityForResult(intent1, 10010);
        });
        findComponentById(ResourceTable.Id_camera).setClickedListener(component -> {
            Intent intent1 = new Intent();
            Operation operation = new Intent.OperationBuilder()
                    .withAction("android.media.action.IMAGE_CAPTURE")
                    .build();
            intent1.setOperation(operation);
            // 输出格式
            intent1.setParam("outputFormat", "JPEG");
            String photoCameraName = System.currentTimeMillis() + "_camera.jpg";
            photoCameraPath = getExternalFilesDir(null) + File.separator + photoCameraName;
            startAbilityForResult(intent1, 10011);
        });
        img = (Image) findComponentById(ResourceTable.Id_image);
//        img.setImageElement();
    }

    @Override
    protected void onAbilityResult(int requestCode, int resultCode, Intent resultData) {
        super.onAbilityResult(requestCode, resultCode, resultData);
        if (requestCode == 10010) {

            String chooseImgUri = resultData.getUriString();
//定义数据能力帮助对象
            DataAbilityHelper helper = DataAbilityHelper.creator(getContext());
            //定义图片来源对象
            ImageSource imageSource = null;
            //获取选择的Img对应的Id
            String chooseImgId = null;
            //如果是选择文件则getUriString结果为content://com.android.providers.media.documents/document/image%3A30，其中%3A是":"的URL编码结果，后面的数字就是image对应的Id
            //如果选择的是图库则getUriString结果为content://media/external/images/media/30，最后就是image对应的Id
            //这里需要判断是选择了文件还是图库
            if (chooseImgUri.lastIndexOf("%3A") != -1) {
                chooseImgId = chooseImgUri.substring(chooseImgUri.lastIndexOf("%3A") + 3);
            } else {
                chooseImgId = chooseImgUri.substring(chooseImgUri.lastIndexOf('/') + 1);
            }
            //获取图片对应的uri，由于获取到的前缀是content，我们替换成对应的dataability前缀
            Uri uri = Uri.appendEncodedPathToUri(AVStorage.Images.Media.EXTERNAL_DATA_ABILITY_URI, chooseImgId);
            HiLog.info(LABEL_LOG, "选择图片dataability路径:" + uri.toString());
            try {
                //读取图片
                FileDescriptor fd = helper.openFile(uri, "r");
                imageSource = ImageSource.create(fd, null);
                //创建位图
                PixelMap pixelMap = imageSource.createPixelmap(null);
                //设置图片控件对应的位图
                img.setPixelMap(pixelMap);
            } catch (Exception e) {
                e.printStackTrace();
            } finally {
                if (imageSource != null) {
                    imageSource.release();
                }
            }
        } else {
            HiLog.info(LABEL_LOG, "照片路径路径:" + photoCameraPath);
            ImageSource imageSource = null;
            File file = new File(photoCameraPath);
            try {
                //读取图片
                imageSource = ImageSource.create(file, null);
                //创建位图
                PixelMap pixelMap = imageSource.createPixelmap(null);
                //设置图片控件对应的位图
                img.setPixelMap(pixelMap);
            } catch (Exception e) {
                e.printStackTrace();
            } finally {
                if (imageSource != null) {
                    imageSource.release();
                }
            }
        }
    }

    @Override
    public void onActive() {
        super.onActive();
    }

    @Override
    public void onForeground(Intent intent) {
        super.onForeground(intent);
    }
}
